--- Fired when the core is enabled
function CoreEnabled(event)
end

--- Fired when entering a zone/area that is enabled and tracked by Raid Watch
-- @param areaID number The area ID number.
function EnteringEnabledZone(event, areaID)
end

--- Fired when leaving a zone/area that is enabled and tracked by Raid Watch
function LeavingEnabledZone(event)
end

--- Fired when a boss mod is loaded
-- @param	mod	string	The name of the mod that was loaded
function BossModLoaded(event, mod)
end

--- Fired when the users enables the config mode
-- @param	enable	boolean	True when config mode is enabled, false when disabled
function ConfigMode(event, enable)
end

--- Fired when a boss is engaged and enabled
-- @param	boss	table The boss object for the boss that was engaged.
-- @param	trigger	string|number	If a string then it's a trigger by Emote or Debug, if number then its triggered by combat with the boss, and the number represets the creature ID.
function BossEngaged(event, boss, trigger)
end

--- Fired when boss is disengaged by wipe or win.
-- @param	boss	table	The boss object for the boss that was disengaged.
-- @param	wipe	boolean	True if it was a wipe, else victory.
-- @param	trigger	string|number	The trigger that cased the victory/wipe. \"combat\" when just ending by getting out of combat. Creature ID when victory by killing a boss. The emote message if triggered by some kind of emote.
function BossDisengaged(event, boss, wipe, trigger)
end

--- Fired when a boss mod that is monitoring some kind of combat trigger is loaded.
-- @param	boss	table The boss object that registered the combat tracking.
function CombatTrackingRegistered(event, boss)
end

--- Fired when a player joins the current party.
-- @param	playerName	string	The name of the player that joined.
function PartyPlayerJoined(event, playerName)
end

--- Fired when a player leaves the current party
-- @param	playerName	string	The name of the player who left the party.
function PartyPlayerLeft(event, playerName)
end

--- Fired when joining a party of any kind.
-- @param partyType	string	Representing the kind of party joined (party or raid)
function PartyJoined(event, partyType)
end

--- Fired when leaving a party of any kind.
-- @param partyType	string	Representing the kind of party that was left (party or raid)
function PartyLeft(event, partyType)
end