local mod = RW:Boss("Icecrown Gunship Battle", "Icecrown Citadel", "The Lower Spire")
local L

mod.BossIcon = [[Interface\Icons\achievement_dungeon_hordeairship]]
mod.SupressGlobalHealthFrames = true

---------------------------
-- Variables
local tFightStart
local wBelow, tBelowCD
local wJetPack
local wJump
local iExp, iVet, iElite
local wVet, wElite

---------------------------
-- Init
function mod:OnRegister()
	L = mod:RegisterLocale("enUS")
	if L then		
		L.YellPullAlliance = "Fire up the engines! We got a meetin' with destiny, lads!"
		L.YellWinAlliance = "Don't say I didn't warn ya, scoundrels! Onward, brothers and sisters!"
		L.JumpTriggerAlliance = "We're taking hull damage, get a battle-mage out here to shut down those cannons!"
		
		L.YellPullHorde = "Rise up, sons and daughters of the Horde! Today we battle a hated enemy of the Horde! LOK'TAR OGAR!"		                  
		L.YellWinHorde = "The Alliance falter. Onward to the Lich King!"
		L.JumpTriggerHorde = "We're taking hull damage, get a sorcerer out here to shut down those cannons!"
		
		L.fight_start = "Battle begins.."
		L.fight_start_opt = "Timer for when the battle starts"
		L.fight_start_group = "Fight Begin"
		
		L.ship_health_frames = "Show health frames for the 2 ships"
		L.health_frames = "Show stack frame for Battle Fury"
		
		L.jetpack_check = "Put Goblin Rocket Pack on!"
		L.jetpack_check_opt = "Missing rocket pack warning"
		L.jetpack_check_group = "Goblin Rocket Pack"
		
		L.jump_warning = "Jump Over!"
		L.jump_warning_opt = "Warning when its time to jump over to the other ship"
	end
	L = mod:GetLocale()
	mod.L = L
	
	mod:SetTrigger(L.YellPullHorde, L.YellPullAlliance, 37215, 37540, 37200)	
	mod:SetWin(L.YellWinHorde, L.YellWinAlliance)
	
	mod:SetDelay(70)
	
	self:AddBoolOption("HealthFrame", L.health_frames, nil, nil, select(3, GetSpellInfo(72306)))
	self:AddBoolOption("ShipsHealthFrames", L.ship_health_frames, nil, nil, [[Interface\Icons\spell_holy_sealofsacrifice]])
	
	tFightStart	= self:CustomTimer(47, L.fight_start, L.fight_start_opt, [[Interface\Icons\Achievement_WorldEvent_Valentine]], L.fight_start_group)
	tBelowCD	= self:CDTimer(37, 69705)
	
	wBelow		= self:CastWarning(69705)
	wJetPack	= self:CustomWarning(L.jetpack_check, L.jetpack_check_opt, [[Interface\Icons\ability_mount_rocketmount]], L.jetpack_check_group)
	wJump 		= self:CustomWarning(L.jump_warning, L.jump_warning_opt, 69705)
	wVet		= self:InformWarning(71193)
	wElite		= self:InformWarning(71195)
	
	iExp		= self:Icon("TRIANGLE", 71188, false, nil, "DIAMOND")
	iVet		= self:Icon("STAR", 71193, false, nil, "CIRCLE")
	iElite		= self:Icon("SKULL", 71195, false, nil, "CROSS")
end

---------------------------
-- Locals

-- Jet pack cheeck
local function JetPackCheck()
	local shirt = GetInventoryItemLink("player", 4)
	if shirt then
		shirt = select(3, shirt:find("item:(%d+):"))
		if shirt ~= "49278" then
			wJetPack:Show()
		end
	end
end

-- Delayed adding of ship health frames
local function AddShipHealthFrames()
	RW.Callbacks:Fire("MPBHealthByID", "boss1", "Health Frames")
	RW.Callbacks:Fire("MPBHealthByID", "boss2", "Health Frames")
end

---------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("Below", 69705)
	self:Track("Fury", 72306, 72307, 69638, 69637, 72308)
	
	self:Track("Experienced", 71188)
	self:Track("Veteran", 71193)
	self:Track("Elite", 71195)	
	
	-- TODO, add alliance mobID for the Below Zero casters
	self:DeathTrack("SourcererDeath", 37116)
	
	self:YellTrack("JumpTrigger", L.JumpTriggerHorde, L.JumpTriggerAlliance)
	
	if self.DB.ShipsHealthFrames then
		self:Schedule("AddShipHealth", 47, AddShipHealthFrames)
	end
	
	furyStacks = 0	
	
	if trigger then
		JetPackCheck()
	end
	
	tFightStart:Start()
end

function mod:Experienced(event, args)
	if event == "SPELL_AURA_APPLIED" then
		iExp:Show(args.dGUID)
	end
end

function mod:Veteran(event, args)
	if event == "SPELL_AURA_APPLIED" then
		iVet:Show(args.dGUID)
		wVet:Show()
	end
end

function mod:Elite(event, args)
	if event == "SPELL_AURA_APPLIED" then
		iElite:Show(args.dGUID)
		wElite:Show()
	end
end

function mod:Below(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		wBelow:Show()
		RW.Callbacks:Fire("MPBHealthByID", args.sGUID)
	
	elseif event == "SPELL_AURA_REMOVED" then
		tBelowCD:Start()
	end
end

function mod:Fury(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if (UnitFactionGroup("player") == "Alliance" and self:CUIDFromGUID(args.dGUID) == 36939) or (UnitFactionGroup("player") == "Horde" and self:CUIDFromGUID(args.dGUID) == 36948) then
			if self.db.profile.HealthFrame then
				RW.Callbacks:Fire("MPBSpellStackCombat", args.dGUID, {72306, 72307, 69638, 69637, 72308}, 20)
			end
		end
		
	elseif event == "SPELL_AURA_REMOVED" then
		RW.Callbacks:Fire("MPBSpellStackHide", args.dGUID)
	end
end

function mod:SourcererDeath(cID, gUID)
	RW.Callbacks:Fire("MPBHealthHideByID", gUID)
end

function mod:JumpTrigger(event, msg, name)
	wJump:Show()
end