----------------------------------
-- Create plugin


local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("DKPInfo")
if not plugin then return end

----------------------------------
-- Locals 

local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-DKPInfo")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local AGU = LibStub("AceGUI-3.0")
local UIF

local db
local Utils = RW.Utils

local dragStart, dragStop
local Window, ConfigWindow
local BestIncommingTag = 0
local RequestCount = 0
local RequestSchedule

local defaults = {
	profile = {
		ValidRanks = {
			[1] = true,
			[2] = true,
			[3] = true,
			[4] = true,
			[5] = true,
			[6] = true,
		},
		FilteredNames = {
		},
		PluginEnabled = true,
		TimeTag = 0,
		TimeTagReadable = 0,
		SourceName = "",
		SortMethod = "Class",
		LastSelectedZone = nil,
		SyncEnabled = true,
		
		Height = 480,
		Width = 400,
		BackgroundColor = {0, 0, 0, 0.5},
		LineColor = {0.1, 0.1, 0.1, 1},
		
		Players = {
		},
		Zones = {
		},
		Items = {
		},
		
		x = 0,
		y = 0,
	}
}

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")	
	
	self:RegisterCallback("DKPShow")
	
	if RW_DKP_DATA then
		self:ProcessRawDKPData()
	end	
	
	self:ApplySettings()
end

function plugin:PluginDisable()
end

function plugin:Reset()
	db = self.db.profile
	self:ApplySettings()	
end

function plugin:ShowWindow(toggle)
	if not Window then
		Window = Utils:CreateWindow("DKPInfo", true, dragStart, dragStop, true)
		Window:SetTitle("eqDKP Standings  -  "..db.TimeTagReadable .." - "..db.SourceName)		
		Window:ShowLeftIcon([[Interface\Icons\Ability_Warrior_ShieldBash]])
		Window:Hide()
		Window:SetAlpha(0)
		
		local b = Window:AddTitleButton("Close", nil, true)	
		b:AddClickHandler( function() 
			plugin:ShowWindow(true)
		end)
		Utils:AddTooltip(b, CLOSE)
		
		b = Window:AddTitleButton("Config", nil, true)
		b:AddClickHandler(function()
			plugin:OpenConfigWindow()
		end)
		Utils:AddTooltip(b, LB.CONFIGURE)
		
		b = Window:AddTitleButton("Refresh", nil, true)
		b:AddClickHandler(function()
			plugin.dirty = true
			plugin:Update()
		end)
		Utils:AddTooltip(b, LB.REFRESH)
		
		self.dirty = true
	end		
	
	if Window:IsShown() and toggle then
		Utils.FadeOut(Window)
		if Window.DKPPoolSelect then
			Window.DKPPoolSelect:Release()
			Window.DKPPoolSelect = nil
			Window.SortMethodSelect:Release()
			Window.SortMethodSelect = nil
			Window.ScrollFrame:ReleaseChildren()
			Window.ScrollFrame:Release()
			Window.ScrollFrame = nil
		end
		return
	end
	Window.DKPPoolSelect = UIF:Dropdown(L.DKP_POOL, db, "LastSelectedZone", plugin.UpdateList)
	Window.SortMethodSelect = UIF:Dropdown(L.SORT_METHOD, db, "SortMethod", plugin.UpdateList)
	
	Window.DKPPoolSelect.frame:SetParent(Window.content)
	Window.SortMethodSelect.frame:SetParent(Window.content)
	
	Window.ScrollFrame = AGU:Create("ScrollFrame")
	Window.ScrollFrame:SetLayout("Flow")
	Window.ScrollFrame.frame:SetParent(Window.content)
	
	self:ApplySettings()
	Window:Show()
	Utils.FadeIn(Window)
	self:Update()
end

function plugin:ApplySettings()
	if Window then
		Window:SetPoint("CENTER", db.x, db.y)
		Window:SetWidth(db.Width)
		Window:SetContentHeight(db.Height)
		Window:SetBackgroundColor(unpack(db.BackgroundColor))
		
		if Window.DKPPoolSelect then
			Window.DKPPoolSelect.frame:SetPoint("TOPLEFT", Window.content, "TOPLEFT", 5, -5)	
			Window.DKPPoolSelect.frame:SetHeight(60)
			Window.DKPPoolSelect.frame:SetWidth((Window:GetWidth() - 10) / 2)
			Window.DKPPoolSelect.frame:Show()
			
			Window.SortMethodSelect.frame:SetPoint("TOPLEFT", Window.DKPPoolSelect.frame, "TOPRIGHT", 5, 0)
			Window.SortMethodSelect.frame:SetPoint("BOTTOMLEFT", Window.DKPPoolSelect.frame, "BOTTOMRIGHT", 5, 0)
			Window.SortMethodSelect.frame:SetPoint("RIGHT", Window.content, -5, 0)
			Window.SortMethodSelect.frame:Show()
			
			Window.ScrollFrame.frame:SetPoint("TOPLEFT", Window.DKPPoolSelect.frame, "BOTTOMLEFT", 0, -5)
			Window.ScrollFrame.frame:SetPoint("BOTTOMRIGHT", Window.content, -5, 5)
			Window.ScrollFrame.frame:Show()
		end
	end
	
	if db.SyncEnabled then
		RW.Com:RegisterListener(self, "RW_DKP_GetTimeTag")
		RW.Com:RegisterListener(self, "RW_DKP_HaveTimeTag")
		RW.Com:RegisterListener(self, "RW_DKP_Data")
		RW.Com:RegisterListener(self, "RW_DKP_GetData")		
	else
		RW.Com:UnregisterAllListeners(self)		
	end
end

-- Comm events ----------------------------------
-------------------------------------------------
do
	function plugin:RW_DKP_GetTimeTag(sender, dist, tag, ...)
		if sender == UnitName("player") then return end
		RW.Com:Send("RW_DKP_HaveTimeTag", "WHISPER", sender, db.TimeTag)	
	end

	function plugin:RW_DKP_HaveTimeTag(sender, dist, tag, ...)
		if sender == UnitName("player") then return end
		if tag > db.TimeTag and tag > BestIncommingTag then
			RW.Com:Send("RW_DKP_GetData", "WHISPER", sender)
		end
	end

	function plugin:RW_DKP_GetData(sender, dist)
		if RequestSchedule then
			self:CancelTimer(RequestSchedule, true)
			RequestSchedule = nil
			RequestSchedule = self:ScheduleTimer("SendData", 5, {dist = "GUILD", target = nil})
		else
			RequestSchedule = self:ScheduleTimer("SendData", 5, {dist = "WHISPER", target = sender})	
		end
	end

	function plugin:RW_DKP_Data(sender, dist, timetag, timereadable, players, zones, items, ...)
		if timetag > db.TimeTag then
			if players then db.Players = players end
			if zones then db.Zones = zones end
			if items then db.Items = items end
			db.TimeTag = timetag
			db.TimeTagReadable = timereadable
			db.SourceName = sender
			self.dirty = true
			self:Update()
		end
	end

	function plugin:SendData(targetData)
		local tosend = {}
		for name, data in pairs(db.Players) do
			local rank, class = plugin:GetRank(name)
			if db.ValidRanks[rank] then
				tosend[name] = data
			end
		end
		RW.Com:Send("RW_DKP_Data", targetData.dist, targetData.target, db.TimeTag, db.TimeTagReadable, tosend, db.Zones)
		RequestCount = 0
	end
end

-- Message events ----------------------------------
----------------------------------------------------
function plugin:DKPShow(event)
	self:ShowWindow(true)
end

-- Update GUI --------------------------------------
----------------------------------------------------
do
	local ZoneList = {}
	local SortList = {
		Name = {value = "Name", text = "Name"},
		Class = {value = "Class", text = "Class"},
		Rank = {value = "Rank", text = "Guild Rank"},
		DKP = {value = "DKP", text = "DKP"},
	}

	function plugin:Update()
		if Window and Window:IsShown() then
			Window:SetTitle(L.NAME .." - "..db.TimeTagReadable .." - "..db.SourceName)
						
			wipe(ZoneList)
			local firstZone
			for index, d in pairs(db.Zones) do
				for zone, data in pairs(d) do
					firstZone = firstZone or zone
					ZoneList[zone] = {value = zone, text = data.disc}
				end
			end
			
			Window.DKPPoolSelect:SetList(ZoneList)
			Window.DKPPoolSelect:SetValue(db.LastSelectedZone or firstZone)
			Window.SortMethodSelect:SetList(SortList)
			Window.SortMethodSelect:SetValue(db.SortMethod)
			
			db.LastSelectedZone = db.LastSelectedZone or firstZone
			
			if db.LastSelectedZone then
				self:UpdateList()
			end
		end
	end

	local NameList = {}
	local SortFunctions = {
		["Name"] = function(a, b)
			return a.name < b.name
		end,
		
		["Rank"] = function(a, b)
			if a.rank == b.rank then
				if a.class == b.class then
					return a.dkp > b.dkp
				end
				return a.class < b.class
			end
			return a.rank < b.rank
		end,
		
		["DKP"] = function(a, b)
			return a.dkp > b.dkp
		end,
		
		["Class"] = function(a, b)
			if a.class == b.class then
				return a.dkp > b.dkp
			end
			return a.class < b.class
		end
	}

	function plugin.UpdateList(self, event)
		if event then plugin.dirty = true end
		local zone = db.LastSelectedZone	
		if not zone or zone == "" then return end		
		local sortMethod = SortFunctions[db.SortMethod]
		
		Window.ScrollFrame:ReleaseChildren()
		
		Window.ScrollFrame:AddChild(UIF:Title3(LB.NAME, nil, nil, nil, nil, nil, 0.33))
		Window.ScrollFrame:AddChild(UIF:Title3(L.RANK, nil, nil, nil, nil, nil, 0.33))
		Window.ScrollFrame:AddChild(UIF:Title3(L.DKP, nil, nil, nil, nil, nil, 0.33))
		Window.ScrollFrame:AddChild(UIF:NewLine(5))
		
		if plugin.dirty then
			wipe(NameList)		
			for name, data in pairs(db.Players) do		
				local rank, class = plugin:GetRank(name)
				if db.ValidRanks[rank] and not db.FilteredNames[name] then
					tinsert(NameList, {rank = rank, class = class, name = name, dkp = data[zone.."_current"]})			
				end
			end		
			plugin.dirty = nil
			table.sort(NameList, sortMethod)
		end
		
		local counter = 0
		for i, data in pairs(NameList) do
			counter = counter + 1
			local c = RAID_CLASS_COLORS[data.class]
			local color = {c.r, c.g, c.b}
			local name = UIF:Text1(data.name, color, nil, nil, nil, nil, 0.33)		
			Window.ScrollFrame:AddChild(name)
			local rank = UIF:Text1(GuildControlGetRankName(data.rank), nil, nil, nil, nil, nil, 0.33)
			Window.ScrollFrame:AddChild(rank)
			local dkp = UIF:Text1(data.dkp, nil, nil, nil, nil, nil, 0.33)
			Window.ScrollFrame:AddChild(dkp)
			if counter % 2 == 0 then
				name:SetBackground(db.LineColor)
				rank:SetBackground(db.LineColor)
				dkp:SetBackground(db.LineColor)
			end
		end
	end
end

function plugin:GetRank(player)
	local name = ""
	local index = 1
	while name do
		name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName = GetGuildRosterInfo(index)
		
		if name == player then
			return rankIndex + 1, classFileName
		end
		
		index = index + 1
	end
end


-- Options -------------------------------
------------------------------------------
do
	function plugin:OpenConfigWindow(toggle)
		if not ConfigWindow then
			ConfigWindow = UIF:Frame(L.NAME)	
			ConfigWindow:SetHeight(800)
			ConfigWindow:SetCallback("OnClose", function()
				ConfigWindow:ReleaseChildren()
				ConfigWindow.frame:Hide()
			end)
			ConfigWindow:SetLayout("Fill")	
			ConfigWindow:Show()
			ConfigWindow:AddChildren(plugin:GetGUI())
			return
		end
		
		if toggle then
			if ConfigWindow.frame:IsShown() then
				ConfigWindow:Hide()
				ConfigWindow:Show()
				ConfigWindow:AddChildren(plugin:GetGUI())
			end		
		else
			if ConfigWindow.frame:IsShown() then
				ConfigWindow:Hide()
			else		
				ConfigWindow:Show()
				ConfigWindow:AddChildren(plugin:GetGUI())		
			end
		end	
	end


	function plugin:GetOptions()
		return LB.PLUGINS, L.NAME, self.GetGUI, [[Interface\Icons\Ability_Warrior_ShieldBash]]
	end

	local function changeCallback()
		plugin:ApplySettings()
	end
	
	-- Locals
	local LastSelectedTab
	local SideList = {
		["LEFT"] = "Left",
		["RIGHT"] = "Right",
	}
	local Ranks = {}
	for i=1, GuildControlGetNumRanks() do
		Ranks[i] = GuildControlGetRankName(i)
	end
	-- Functions
	local TabSelected

	function plugin:GetGUI()
		local self = plugin		
		
		local scroll = AGU:Create("ScrollFrame")	
		scroll:SetLayout("Flow")
		
		label = UIF:MainTitleLabel(L.NAME)
		scroll:AddChild(label)	
		
		-- Information
		local infogroup = UIF:InlineGroup1()
		local label = UIF:Text1(L.DESC)
		label:SetImage([[Interface\Icons\Ability_Warrior_ShieldBash]])
		infogroup:AddChild(label)
		scroll:AddChild(infogroup)
		
		scroll:AddChild(UIF:CheckBox(L.ENABLE_SYNC, db, "SyncEnabled", function() 
			RW.Com:Send("RW_DKP_HaveTimeTag", "GUILD", nil, db.TimeTag)	
			RW.Com:Send("RW_DKP_GetTimeTag", "GUILD", nil, db.TimeTag)
		end, L.ENABLE_SYNC_DESC, 0.5))
		scroll:AddChild(UIF:Button(LB.REFRESH, function() plugin:Update() end))
		scroll:AddChild(UIF:Button(LB.SHOW, function() plugin:ShowWindow() end))
		
		local group = UIF:InlineGroup1()
		scroll:AddChild(group)
		
		group:AddChild(UIF:Text1(L.VISIBLE_RANKS_DESC, nil, nil, nil, nil, nil, 0.6))
		group:AddChild(UIF:MultiSelectDropdown(L.VISIBLE_RANKS, db.ValidRanks, nil, Ranks, 0.5))
		group:AddChild(UIF:NewLine())
		group:AddChild(UIF:ColorSelect(L.LINE_COLOR, db, "LineColor", changeCallback, true, 0.5))		
		group:AddChild(UIF:ColorSelect(LB.BACKGROUND_COLOR, db, "BackgroundColor", changeCallback, true, 0.5))
		group:AddChild(UIF:Slider(LB.WIDTH, db, "Width", 50, 1000, 1, changeCallback, 0.5))
		group:AddChild(UIF:Slider(LB.HEIGHT, db, "Height", 60, 1000, 1, changeCallback, 0.5))						
		
		local button = UIF:Button(LB.RESET, function()
			wipe(db.Players)
			wipe(db.Zones)
			wipe(db.Items)
			db.TimeTag = 0
			db.TimeTagReadable = 0
			db.SourceName = ""
			db.LastSelectedZone = nil
			if Window then
				Window.ListGroup:ReleaseChildren()
				plugin:Update()
			end
			plugin.db:ResetProfile() 
		end)
		button:SetUserData("confirm", true)
		button:SetUserData("TriggerGlobalUpdate", true)
		scroll:AddChild(button)
		
		scroll:FixScroll()
		
		return scroll
	end
end	

-- Moving ------------------------------------
----------------------------------------------
dragStart = function(self)
	Window:StartMoving()
end

dragStop = function(self)
	Window:StopMovingOrSizing()
	local scale = Window:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = Window:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	x = x * scale
	y = y * scale	
	
	db.x, db.y = x, y
	
	if ConfigWindow then
		plugin:OpenConfigWindow(true)
	end
end


-- Process Data ------------------------------
----------------------------------------------
function plugin:ProcessRawDKPData()
	if db.TimeTag < tonumber(RW_DKP_DATA.DKPInfo.timestamp) then
		db.TimeTag = tonumber(RW_DKP_DATA.DKPInfo.timestamp)
		db.TimeTagReadable = RW_DKP_DATA.DKPInfo.date
		db.SourceName = UnitName("player")
			
		if RW_DKP_DATA.gdkp and RW_DKP_DATA.gdkp.players then
			wipe(db.Players)
			for player, dkpdata in pairs(RW_DKP_DATA.gdkp.players) do
				local playerdb = db.Players[player] or {}
				
				-- Only save current DKP
				for i, data in pairs(dkpdata) do
					if i:find(".*_current") then
						playerdb[i] = data
					end
				end
				
				db.Players[player] = playerdb
			end
		end	
		
		if RW_DKP_DATA.multiTable then
			wipe(db.Zones)
			for zone, data in pairs(RW_DKP_DATA.multiTable) do
				local zonedb = db.Zones[zone] or {}
				
				for i, zonedata in pairs(data) do
					zonedb[i] = zonedata
				end
				
				db.Zones[zone] = zonedb
			end
		end
	end
	RW_DKP_DATA = nil
end
