----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("RaidSpecs")
if not plugin then return end

----------------------------------
-- Locals 
local LGT = LibStub("LibGroupTalents-1.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-RaidSpecs")
local LB = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")
local LBB = LibStub("AceLocale-3.0"):GetLocale("RW2-Base")
local LSM = LibStub("LibSharedMedia-3.0")
local AGU = LibStub("AceGUI-3.0")
local UIF

local db
local Utils = RW.Utils
local PluginIcon = [[Interface\Icons\Achievement_Reputation_08]]

local Window, ConfigWindow, Anchor
local dragStart, dragStop
local Party

local defaults = {
	profile = {
		PluginEnabled = true,
		x = 0, 
		y = 0,
		Width = 300,	

		NotifyRoleChange = false,
		NotifySpecChange = false,
	}
}

----------------------------------
-- Init
function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
	
	LGT.RegisterCallback(self, "LibGroupTalents_RoleChange", "RoleChanged")
	LGT.RegisterCallback(self, "LibGroupTalents_Update", "SpecUpdate")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end
	db.SpecTable = {}
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")
	
	Party = RW.Party
	
	self:Reset()
end

function plugin:PluginDisable()
end

function plugin:Reset()
	db = self.db.profile
end

-- Events ------------------------------------
----------------------------------------------
function plugin:RoleChanged(event, guid, unit, newRole, oldRole)
	db.SpecTable[guid] = db.SpecTable[guid] or {}
	db.SpecTable[guid].role = newRole
	if db.NotifyRoleChange and oldRole then
		RW.Callbacks:Fire("MessageShow", UnitName(unit).." changed from "..LBB[string.upper(oldRole)].." to "..LBB[string.upper(newRole)], nil, nil, nil, nil, db.SpecTable[guid].icon)
	end
end

function plugin:SpecUpdate(event, guid, unit, spec, tree1, tree2, tree3, oldSpec)
	local highTree = tree1
	local treeNumber = 1
	if tree2 > highTree then highTree = tree2 treeNumber = 2 end
	if tree3 > highTree then highTree = tree3 treeNumber = 3 end		
	
	db.SpecTable[guid] = db.SpecTable[guid] or {}
	db.SpecTable[guid].spec = spec
	db.SpecTable[guid].tree = treeNumber
	db.SpecTable[guid].icon = select(treeNumber, LGT:GetTreeIcons(select(2, UnitClass(unit))))
	
	if oldSpec and spec ~= oldSpec and db.NotifySpecChange then
		self:Msg(UnitName(unit).." speced "..spec.." from "..oldSpec)
	end
end


-- UI ----------------------------------------
----------------------------------------------
do
	function plugin:GetOptions()
		return LB.PLUGINS, L.NAME, self.GetGUI, PluginIcon
	end
	
	
	function plugin:GetGUI()
		self = plugin
		
		local scroll = AGU:Create("ScrollFrame")
		scroll:SetLayout("Flow")
		
		local label = UIF:MainTitleLabel(L.NAME)
		scroll:AddChild(label)	
		
		-- Information
		local infogroup = UIF:InlineGroup1()
		label = UIF:Text1(L.DESC)
		label:SetImage(PluginIcon)
		infogroup:AddChild(label)
		scroll:AddChild(infogroup)
		
		local group = UIF:InlineGroup()
		scroll:AddChild(group)
		
		group:AddChild(UIF:CheckBox("Notify role change", db, "NotifyRoleChange", nil, "Will notify when someone in the raid/party changes spec to a different role, eg. Tank -> DPS", 0.5))
		group:AddChild(UIF:CheckBox("Notify spec change", db, "NotifySpecChange", nil, "Will notify whenever someone changes spec, eg. Unholy -> Frost", 0.5))
		
		return scroll
	end
end